/*****************************************************************************
 *                        Web3d.org Copyright (c) 2001
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// Standard imports
import java.util.HashMap;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

// Local imports
// None

/**
 * An OpenGL LineArray.
 * <p>
 *
 * Lines cannot be picked in the traditional manner using geometry-based
 * picking. The only way to pick them is based on their bounds.
 *
 * @author Justin Couch
 * @version $Revision: 1.4 $
 */
public class IndexedLineArray extends IndexedVertexGeometry
{
    /**
     * Constructs a LineArray with default values.
     */
    public IndexedLineArray()
    {
    }

    /**
     * Issue ogl commands needed for this renderable object.
     *
     * @param gl The gl context to draw with
     * @param glu The utility context to draw with
     */
    public void renderState(GL gl, GLU glu)
    {
        // No coordinates, do nothing.
        if((vertexFormat & COORDINATE_MASK) == 0)
            return;

        //  State enable first
        int vtx_size = vertexFormat & 0x07;
        gl.glEnableClientState(GL.GL_VERTEX_ARRAY);
        gl.glVertexPointer(vtx_size, GL.GL_FLOAT, 0, vertexBuffer);

        if((vertexFormat & NORMALS) != 0)
        {
            gl.glEnableClientState(GL.GL_NORMAL_ARRAY);
            gl.glNormalPointer(GL.GL_FLOAT, 0, normalBuffer);
        }

        if((vertexFormat & TEXTURE_MASK) != 0)
        {
            // single texturing or multi-texturing
            if(numTextureSets == 1)
            {
                gl.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY);
                gl.glTexCoordPointer(textureTypes[0],
                                     GL.GL_FLOAT,
                                     0,
                                     textureBuffer[0]);
            }
            else
            {
                for(int i = 0; i < numTextureSets; i++)
                {
                    int set_id = textureSets[i];
                    gl.glClientActiveTexture(GL.GL_TEXTURE0 + i);
                    gl.glEnableClientState(GL.GL_TEXTURE_COORD_ARRAY);
                    gl.glTexCoordPointer(textureTypes[set_id],
                                         GL.GL_FLOAT,
                                         0,
                                         textureBuffer[set_id]);
                }
            }
        }

        if((vertexFormat & COLOR_MASK) != 0)
        {
            gl.glEnableClientState(GL.GL_COLOR_ARRAY);
            int size = ((vertexFormat & COLOR_3) != 0) ? 3 : 4;
            gl.glColorPointer(size, GL.GL_FLOAT, 0, colorBuffer);
        }

        if((vertexFormat & COLOR2_MASK) != 0)
        {
            gl.glEnableClientState(GL.GL_SECONDARY_COLOR_ARRAY);
            gl.glSecondaryColorPointer(3, GL.GL_FLOAT, 0, color2Buffer);
        }

        if((vertexFormat & FOG_MASK) != 0)
        {
            gl.glFogi(GL.GL_FOG_COORDINATE_SOURCE, GL.GL_FOG_COORDINATE);
            gl.glEnableClientState(GL.GL_FOG_COORDINATE_ARRAY);
            gl.glFogCoordPointer(GL.GL_FLOAT, 0, fogBuffer);
        }

        gl.glDrawElements(GL.GL_LINES,
                          numIndices,
                          GL.GL_UNSIGNED_INT,
                          indices);

        if((vertexFormat & FOG_MASK) != 0)
        {
            gl.glDisableClientState(GL.GL_FOG_COORDINATE_ARRAY);
            gl.glFogi(GL.GL_FOG_COORDINATE_SOURCE, GL.GL_FRAGMENT_DEPTH);
        }

        if((vertexFormat & COLOR2_MASK) != 0)
        {
            gl.glDisableClientState(GL.GL_SECONDARY_COLOR_ARRAY);
            gl.glSecondaryColor3f(1f,1f,1f);
        }

        if((vertexFormat & COLOR_MASK) != 0)
        {
            gl.glDisableClientState(GL.GL_COLOR_ARRAY);
            gl.glColor4f(1f,1f,1f,1f);
        }

        if((vertexFormat & TEXTURE_MASK) != 0)
        {
            // single texturing or multi-texturing
            if(numTextureSets == 1)
            {
                gl.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY);
            }
            else
            {
                for(int i = numTextureSets - 1; i >= 0; i--)
                {
                    gl.glClientActiveTexture(GL.GL_TEXTURE0 + i);
                    gl.glDisableClientState(GL.GL_TEXTURE_COORD_ARRAY);
                }
            }
        }

        if((vertexFormat & NORMALS) != 0)
            gl.glDisableClientState(GL.GL_NORMAL_ARRAY);

        gl.glDisableClientState(GL.GL_VERTEX_ARRAY);
    }

    /**
     * Restore all openGL state.
     *
     * @param gl The gl context to draw with
     * @param glu The utility context to draw with
     */
    public void restoreState(GL gl, GLU glu)
    {
    }
}