/*****************************************************************************
 *                        Web3d.org Copyright (c) 2001
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// Standard imports
// None

// Application specific imports
// None

/**
 * Abstract representation of a piece of code that wants to know about when
 * a scene graph node is requiring an update.
 *
 * @author Justin Couch
 * @version $Revision: 1.8 $
 */
public interface NodeUpdateHandler
{
    /**
     * Check to see if writing to the bounding information of the node is
     * permitted currently.
     *
     * @param src The object that is requesting the check
     * @return true if the end user can write, false if not
     */
    public boolean isBoundsWritePermitted(Object src);

    /**
     * Check to see if writing to the data information of the node is
     * permitted currently.
     *
     * @param src The object that is requesting the check
     * @return true if the end user can write, false if not
     */
    public boolean isDataWritePermitted(Object src);

    /**
     * Check to see if picking is permitted currently.
     *
     * @return true if the end user can pick, false if not
     */
    public boolean isPickingPermitted();

    /**
     * Notify the handler that you have updates to the SG that might alter
     * a node's bounds.
     *
     * @param l The change requestor
     * @param src The object that is passing this listener through.
     * @param intL Internal listener for making callbacks at a later time
     *    to propogate the bounds changes.
     * @return Was the notification accepted.  Duplicates will return false.
     */
    public boolean boundsChanged(NodeUpdateListener l,
                              Object src,
                              InternalNodeUpdateListener intL);

    /**
     * Notify the handler that you have updates to the SG that will not
     * alter a node's bounds.
     *
     * @param l The change requestor
     * @param src The object that is passing this listener through.
     */
    public void dataChanged(NodeUpdateListener l, Object src);

    /**
     * The object provided is no longer needed by its parent. Put this on
     * a queue for objects that could potentially be deleted next frame. Used
     * for any scene graph object type that must be explicitly cleaned up
     * during the OpenGL render cycle, such as textures and pBuffers.
     *
     * @param obj The object that is registering itself for deletion
     */
    public void objectRemoved(DeletableSceneGraphObject obj);

    /**
     * Get the picking handler so that we can do some picking operations.
     *
     * @return the current instance of the picking system
     */
    public PickingHandler getPickHandler();
}